;NCID Windows Client installer
;Written by Rich West 01/27/2006
;Updated by Lyman Epp 02/11/2006
;rewritten by John Chmielewski 07/09/2017
;last edit: 5/10/2024 by jlc

;--------------------------------
;Include Modern UI

    !include "MUI2.nsh"

    !include nsDialogs.nsh
    !include LogicLib.nsh
    !include WordFunc.nsh
    !include FileFunc.nsh

;--------------------------------
;Application

    !define NAME            "ncid"
    !define FULL_NAME       "NCID Universal Client"
    !define PUBLISHER       "NCID Project"
    !define VERSION         "XxXxX"
    !define PROG            "${NAME}.tcl"
    !define TZSCRIPT        "phonetz"
    !define LOGO            "logo.png"
    !define NEWCONF         "ncid.newconf"
    !define MESSAGE_DIALOG  "message_dialog"
    !define US_NUMBER_INFO  "us_number_info"
    !define DISPLAY_NCID_VARIABLES "display_ncid_variables"

    !define IMAGES          "$INSTDIR\images"
    !define ICONS           "$INSTDIR\icons"
    !define PLUGINS         "$INSTDIR\plugins"
    !define MSGS            "$INSTDIR\msgs"
    !define LOGS            "$INSTDIR\logs"
    !define LIB             "$INSTDIR\lib"
    !define DOTNCID         "$INSTDIR\.ncid"

    !define LICENSE_FILE    "GPL.txt"
    !define WEB_TITLE       "NCID Project Web Page"
    !define WEB_PAGE        "http://ncid.sourceforge.net"
    !define SPLASH_IMAGE    "ncid_splash.bmp"
    !define INSTALLER_NAME  "${NAME}-${VERSION}-client_windows_setup.exe"
    !define IPADDR2CHG      "localhost"
    !define IPADDRDEFAULT   "127.0.0.1"
    !define IPPORT          "3333"

    !define ROOT_KEY HKLM
    !define INSTALLER_KEY   "SOFTWARE\${NAME}"
    !define UNINSTALL_KEY \
        "SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\${NAME}"
    ;!define AUTORUN_KEY    "SOFTWARE\Microsoft\Windows\CurrentVersion\Run"

    !define TKAPP  "wish.exe"
    ; older versions of ActiveTCL used wiish86.exe
    !define OTKAPP "wish86.exe"

    !define Line1  "Kept previous ncid.conf$\r$\n"
    !define Line2  "Created ${NEWCONF}$\r$\n"
    !define Line3  "Edit ${NEWCONF} and rename it to ncid.conf$\r$\n"

;--------------------------------
;General

    Name    "${FULL_NAME}"
    OutFile "${INSTALLER_NAME}"
    Icon    "${NAME}.ico"

    SetCompressor lzma

    ;WindowIcon On

    ; Default installation folder
    ;InstallDir "$PROGRAMFILES\${NAME}"
    InstallDir "C:\${NAME}"
    AllowRootDirInstall true

    ; Get installation folder from registry if available
    InstallDirRegKey "${ROOT_KEY}" ${INSTALLER_KEY} ""

;--------------------------------
;Variables

    Var StartMenuFolder
    Var Dialog
    Var Label
    Var Label2
    Var Label3
    Var Text
    Var AddrPort
    Var Number
    Var Verbose
    Var CheckBox
    Var Skipit

;--------------------------------
!macro ReplaceInFile OLD_STR FST_OCC NR_OCC REPLACEMENT_STR FILE_TO_MODIFIED
    ;From AdvReplaceInFile.nsh

  Push "${OLD_STR}" ;text to be replaced
  Push "${REPLACEMENT_STR}" ;replace with
  Push "${FST_OCC}" ; starts replacing onwards FST_OCC occurrences
  Push "${NR_OCC}" ; replaces NR_OCC occurrences in all
  Push "${FILE_TO_MODIFIED}" ; file to replace in
  Call AdvReplaceInFile

!macroend

;--------------------------------
!macro UninstallApp
  ; http://nsis.sourceforge.net/Auto-uninstall_old_before_installing_new
  ReadRegStr $R0 ${ROOT_KEY} "${UNINSTALL_KEY}" "UninstallString"
  StrCmp $R0 "" done

  MessageBox MB_OKCANCEL|MB_ICONEXCLAMATION \
  "${NAME} is already installed. \
   $\n$\nClick 'OK' to upgrade \
   $\nClick `Cancel` to cancel this upgrade." \
  IDOK uninst
  Abort

;Run the uninstaller
uninst:
  ClearErrors
  ExecWait '$R0 /S _?=$INSTDIR'

done:
!macroend

;--------------------------------
!macro CheckUserRights
  ClearErrors
  UserInfo::GetName
  IfErrors good
  Pop $0
  UserInfo::GetAccountType
  Pop $1
  StrCmp $1 "Admin" good
  StrCmp $1 "Power" good

  MessageBox MB_OK "Administrative rights are needed for that."
  Abort

good:
!macroend

;--------------------------------
Function .onInit
    !insertmacro CheckUserRights
    StrCpy $AddrPort "${IPADDRDEFAULT}:${IPPORT}"
    StrCpy $Verbose "1"
    !insertmacro UninstallApp
FunctionEnd

;--------------------------------
Function un.onInit
    !insertmacro CheckUserRights
FunctionEnd

;--------------------------------
Function LaunchScript
    ExecShell "open" "$INSTDIR\${PROG}"
FunctionEnd

;--------------------------------
Function configModify

    !insertmacro MUI_HEADER_TEXT $(CHANGE_TITLE) $(CHANGE_SUBTITLE)

    nsDialogs::Create 1018
    Pop $Dialog

    ${If} $R0 == error
        Abort
    ${EndIf}

    ${NSD_CreateLabel} 0u 10% 50% 12u "Enter <address:port | address | :port>:"
    Pop $Label
    ${NSD_CreateText} 60% 10% 100% 12u $AddrPort
    Pop $Text

    ${NSD_OnChange} $Text changeServerAddress
    ${NSD_SetFocus} $Text
    SendMessage $Text ${EM_SETSEL} 0 -1

    ${NSD_CreateLabel} 0u 30% 50% 12u "Enter Verbose level (range 1-9):"
    Pop $Label2
    ${NSD_CreateNumber} 60% 30% 4% 12u $Verbose
    Pop $Number

    ${NSD_OnChange} $Number changeVerbose
    ;${NSD_SetFocus} $Number
    SendMessage $Number ${EM_SETSEL} 0 -1

    nsDialogs::Show

FunctionEnd

;--------------------------------
Function changeVerbose

    ${NSD_GetText} $Number $Verbose

    ${If} $Verbose < 1
        ; $Verbose must be greater than 0
        StrCpy $Verbose "1"
    ${ElseIf} $Verbose > 9
        ; $Verbose must be < 10
        StrCpy $Verbose "9"
    ${EndIf}

FunctionEnd

;--------------------------------
Function changeServerAddress

    ${NSD_GetText} $Text $AddrPort

    ${If} $AddrPort == ""
        ; $AddrPort must have some text
        StrCpy $AddrPort "${IPADDRDEFAULT}:${IPPORT}"
    ${EndIf}

    ${WordFind} $AddrPort ":" "#" $R0
    ${IfNot} $R0 == "2"
        ; $AddrPort is not address:port
        ${WordFind} $AddrPort ":" "*" $R0
        ${If} $R0 == "1"
        ; $AddrPort == :port, add default address
            StrCpy $AddrPort "${IPADDRDEFAULT}$AddrPort"
        ${Else}
        ; $AddrPort == address, add default port
            StrCpy $AddrPort "$AddrPort:${IPPORT}"
        ${EndIf}
    ${EndIf}

FunctionEnd

;--------------------------------
Function configUpdate

    StrCmp "$AddrPort" "${IPADDRDEFAULT}:${IPPORT}" +2 0
        !insertmacro ReplaceInFile "# set Hosts $\"${IPADDR2CHG}:${IPPORT}$\"" 1 1 "set Hosts $\"$AddrPort$\"" "$INSTDIR\${NEWCONF}"

    ${If} $Verbose != 1
        !insertmacro ReplaceInFile "# set Verbose 0" 1 1 "set Verbose $Verbose" "$INSTDIR\${NEWCONF}"
    ${EndIf}

    ; if ${NAME}.conf exists, do not replace it
    IfFileExists $INSTDIR\${NAME}.conf 0 +3
        MessageBox MB_OK "${Line1}${Line2}${Line3}"
        Abort

    Rename $INSTDIR\${NAME}.newconf $INSTDIR\${NAME}.conf

FunctionEnd

;--------------------------------
Function configVerify

    !insertmacro MUI_HEADER_TEXT $(VERIFY_TITLE) $(VERIFY_SUBTITLE)

    nsDialogs::Create 1018
    Pop $Dialog

    ${If} $R0 == error
        Abort
    ${EndIf}

    ${NSD_CreateLabel} 0 10% 100% 12u "Server address will be set to: $AddrPort"
    Pop $Label2

    ${NSD_CreateLabel} 0 20% 100% 12u "Verbose will be set to:  $Verbose"
    Pop $Label

    ${NSD_CreateLabel} 0 30% 100% 12u "After installation, edit ncid.conf to change any variables"
    Pop $Label3

    nsDialogs::Show

    ;MessageBox MB_OK "${Verify1}${Verify2}"

FunctionEnd

;--------------------------------
Function AdvReplaceInFile

/*
    From AdvReplaceInFile.nsh
    Written by Stu (A frow UK) for modifying batch files.
    https://nsis.sourceforge.io/More_advanced_replace_text_in_file

    Version: 2020-12-12
        Mild alterations to make it work, and such as it was intended to in the first place. Pascal Dor.
        bug: scans the read line from right to left
        bug: doesn't do the job unless: FST_OCC = NR_OCC = all, and even so not fully.

    First occurrence to be replaced: FST_OCC.
       FST_OCC = all, renders the same as FST_OCC = 1.
       if FST_OCC greater than the number of occurences in the file: no alteration of the file,
       FST_OCC negative or 0 will leave the file content unchanged no matter the NR_OCC value.
    Nr max of occurrences replaced onwards: NR_OCC,
       if NR_OCC = all --> replacement as long as a string to be replaced is found,
       if NR_OCC = stritly positive integer, replaces up to NR_OCC occurrences provided they exist,
       NR_OCC negative or 0 yields the same as all.

    Order to run down and search the file: from left to right and top down.
    REPLACEMENT_STR, OLD_STR, read line should be less than 1024 characters long.
    For NSIS Unicode, FILE_TO_MODIFIED must be utf-8 encoded.

    Var /Global OLD_STR
    Var /Global FST_OCC
    Var /Global NR_OCC
    Var /Global REPLACEMENT_STR
    Var /Global FILE_TO_MODIFIED

    !macro ReplaceInFile OLD_STR FST_OCC NR_OCC REPLACEMENT_STR FILE_TO_MODIFIED

        Push "${OLD_STR}" ;text to be replaced
        Push "${REPLACEMENT_STR}" ;replace with
        Push "${FST_OCC}" ; starts replacing onwards FST_OCC occurrences
        Push "${NR_OCC}" ; replaces NR_OCC occurrences in all
        Push "${FILE_TO_MODIFIED}" ; file to replace in
        Call AdvReplaceInFile

    !macroend

    Function AdvReplaceInFile
*/
    Exch $0  ;FILE_TO_MODIFIED file to replace in
    Exch
    Exch $1  ;the NR_OCC of OLD_STR occurrences to be replaced.
    Exch
    Exch 2
    Exch $2  ;FST_OCC: the first occurrence to be replaced and onwards
    Exch 2
    Exch 3
    Exch $3  ;REPLACEMENT_STR string to replace with
    Exch 3
    Exch 4
    Exch $4  ;OLD_STR to be replaced
    Exch 4
    Push $5  ;incrementing counter
    Push $6  ;a chunk of read line
    Push $7  ;the read line altered or not
    Push $8  ;left string
    Push $9  ;right string or forster read line
    Push $R0 ;temp file handle
    Push $R1 ;FILE_TO_MODIFIED file handle
    Push $R2 ;a line read
    Push $R3 ;the length of OLD_STR
    Push $R4 ;counts reaching of FST_OCC
    Push $R5 ;counts reaching of NR_OCC
    Push $R6 ;temp file name


    GetTempFileName $R6

    FileOpen $R1 $0 r             ;FILE_TO_MODIFIED file to search in
    FileOpen $R0 $R6 w            ;temp file
    StrLen $R3 $4                 ;the length of OLD_STR
    StrCpy $R4 0                  ;counter initialization
    StrCpy $R5 -1                 ;counter initialization

    loop_read:
     ClearErrors
     FileRead $R1 $R2             ;reading line
     IfErrors exit                ;when end of file has been reached

       StrCpy $5 -1               ;cursor, start of read line chunk
       StrLen $7 $R2              ;read line length
       IntOp $5 $5 - $7           ;cursor initialization
       StrCpy $7 $R2              ;$7 contains read line

    loop_filter:
       IntOp $5 $5 + 1            ;cursor shifting
       StrCmp $5 0 file_write     ;end of line has been reached
       StrCpy $6 $7 $R3 $5        ;a chunk of read line of length OLD_STR
       StrCmp $6 $4 0 loop_filter ;continues to search OLD_STR if no match

    StrCpy $8 $7 $5               ;left part
    IntOp $6 $5 + $R3
    IntCmp $6 0 yes no            ;left part + OLD_STR == full line read ?
    yes:
    StrCpy $9 ""
    Goto done
    no:
    StrCpy $9 $7 "" $6            ;right part
    done:
    StrCpy $9 $8$3$9              ;replacing OLD_STR by REPLACEMENT_STR in forster read line

    IntOp $R4 $R4 + 1             ;counter incrementation
    ;MessageBox MB_OK|MB_ICONINFORMATION \
    ;"count R4 = $R4, fst_occ = $2"
    StrCmp $2 all follow_up       ;exchange ok, then goes to search the next OLD_STR
    IntCmp $R4 $2 follow_up       ;no exchange until FST_OCC has been reached,
    Goto loop_filter              ;and then searching for the next OLD_STR

    follow_up:
    IntOp $R4 $R4 - 1             ;now counter is to be stuck to FST_OCC

    IntOp $R5 $R5 + 1             ;counter incrementation
    ;MessageBox MB_OK|MB_ICONINFORMATION \
    ;"count R5 = $R5, nbr_occ = $1"
    StrCmp $1 all exchange_ok     ;goes to exchange OLD_STR with REPLACEMENT_STR
    IntCmp $R5 $1 finalize        ;proceeding exchange until NR_OCC has been reached

    exchange_ok:
    IntOp $5 $5 + $R3             ;updating cursor
    StrCpy $7 $9                  ;updating read line with forster read line
    Goto loop_filter              ;goes searching the same read line

    finalize:
    IntOp $R5 $R5 - 1             ;now counter is to be stuck to NR_OCC

    file_write:
     FileWrite $R0 $7             ;writes altered or unaltered line
    Goto loop_read                ;reads the next line

    exit:
      FileClose $R0
      FileClose $R1

    ;SetDetailsPrint none
    Delete $0
    Rename $R6 $0                 ;superseding FILE_TO_MODIFIED file with
                                  ;temp file built with REPLACEMENT_STR
    ;Delete $R6
    ;SetDetailsPrint lastused

    Pop $R6
    Pop $R5
    Pop $R4
    Pop $R3
    Pop $R2
    Pop $R1
    Pop $R0
    Pop $9
    Pop $8
    Pop $7
    Pop $6
    Pop $5
    ;These values are stored in the stack in the reverse order they were pushed
    Pop $0
    Pop $1
    Pop $2
    Pop $3
    Pop $4

FunctionEnd

;--------------------------------
Function un.leaveConfig

    nsDialogs::Create 1018
    Pop $Dialog

    ${If} $R0 == error
        Abort
    ${EndIf}

    ${NSD_CreateCheckBox} 0 10% 100% 12u "Remove configuration files, logs and user plugins."
    Pop $CheckBox
    GetFunctionAddress $0 un.CheckBox
    nsDialogs::onClick $CheckBox $0

    nsDialogs::Show

FunctionEnd

;--------------------------------
Function un.CheckBox

    Pop $0 # HWND
    ${NSD_GetState} $CheckBox $Skipit

FunctionEnd

;--------------------------------
;Interface Settings

    !define MUI_ABORTWARNING

    #!define MUI_ICON "${NAME}.ico"

    ; where to store Start Menu directory name
    !define MUI_STARTMENUPAGE_REGISTRY_ROOT "${ROOT_KEY}"
    !define MUI_STARTMENUPAGE_REGISTRY_KEY "${INSTALLER_KEY}"
    !define MUI_STARTMENUPAGE_REGISTRY_VALUENAME "Start Menu"

    ;!define MUI_FINISHPAGE_TEXT_LARGE
    !define MUI_FINISHPAGE_TEXT "Be sure ActiveTCL has been installed before continuing."
    !define MUI_FINISHPAGE_LINK "Click here to download ActiveTCL"
    !define MUI_FINISHPAGE_LINK_COLOR 0000FF
    !define MUI_FINISHPAGE_LINK_LOCATION "https://www.activestate.com/products/tcl/"

    !define MUI_FINISHPAGE_RUN ""
    !define MUI_FINISHPAGE_RUN_PARAMETERS ""
    !define MUI_FINISHPAGE_RUN_FUNCTION "LaunchScript"

    !define MUI_HEADERIMAGE
    !define MUI_HEADERIMAGE_BITMAP ${SPLASH_IMAGE}
    !define MUI_HEADERIMAGE_UNBITMAP ${SPLASH_IMAGE}

;--------------------------------
;Pages

    !insertmacro MUI_PAGE_WELCOME
    !insertmacro MUI_PAGE_LICENSE ${LICENSE_FILE}
    !insertmacro MUI_PAGE_COMPONENTS
    !insertmacro MUI_PAGE_DIRECTORY
    !insertmacro MUI_PAGE_STARTMENU Application $StartMenuFolder
    Page custom configModify
    Page custom configVerify
    !insertmacro MUI_PAGE_INSTFILES
    Page custom configUpdate
    !insertmacro MUI_PAGE_FINISH

    !insertmacro MUI_UNPAGE_WELCOME
    !insertmacro MUI_UNPAGE_CONFIRM
    UninstPage custom un.leaveConfig
    !insertmacro MUI_UNPAGE_INSTFILES
    !insertmacro MUI_UNPAGE_FINISH

;--------------------------------
;Languages

    !insertmacro MUI_LANGUAGE "English"

;--------------------------------
;Installer Sections

; required, cannot select
Section "${NAME} (Required)" Section1

    SectionIn RO

    SetOutPath "$INSTDIR"
    SetShellVarContext all

    ; Put files there
    File "${PROG}"
    File "${NEWCONF}"
    File "${NAME}.ico"
    File "${TZSCRIPT}"
    File "${LICENSE_FILE}"
    File "${LOGO}"

    CreateDirectory "${IMAGES}"
    SetOutPath "${IMAGES}"
    File /nonfatal /a /r "images\"  ;note back slash at the end

    CreateDirectory "${ICONS}"
    SetOutPath "${ICONS}"
    File /nonfatal /a /r "icons\"   ;note back slash at the end

    CreateDirectory "${LIB}"
    SetOutPath "${LIB}"
    File /nonfatal /a /r "lib\"     ;note back slash at the end

    CreateDirectory "${LOGS}"
    SetOutPath "${LOGS}"

    CreateDirectory "${MSGS}"
    SetOutPath "${MSGS}"
    File /nonfatal /a /r "msgs\"    ;note back slash at the end

    CreateDirectory "${PLUGINS}"
    SetOutPath "${PLUGINS}"
    File /nonfatal /a /r "plugins\" ;note back slash at the end

    ; reset output directory to the start directory before creating shortcuts
    SetOutPath "$INSTDIR"

    ; Write the installation path into the registry
    WriteRegStr "${ROOT_KEY}" ${INSTALLER_KEY} "" "$INSTDIR"

    ; Run at logon for all users
    ; WriteRegStr "${ROOT_KEY}" ${AUTORUN_KEY} "${NAME}" "$INSTDIR\${PROG}"

    ; https://nsis.sourceforge.io/Add_uninstall_information_to_Add/Remove_Programs
    ; Write the uninstall keys for Windows
    WriteRegStr "${ROOT_KEY}" ${UNINSTALL_KEY} "DisplayName" "${FULL_NAME}"
    WriteRegStr "${ROOT_KEY}" ${UNINSTALL_KEY} "Publisher" "${PUBLISHER}"
    WriteRegStr "${ROOT_KEY}" ${UNINSTALL_KEY} "DisplayVersion" "${VERSION}"
    WriteRegStr "${ROOT_KEY}" ${UNINSTALL_KEY} "DisplayIcon" "$INSTDIR\${NAME}.ico"
    WriteRegStr "${ROOT_KEY}" ${UNINSTALL_KEY} "UninstallString" \
        '"$INSTDIR\uninstall.exe"'
    WriteRegStr "${ROOT_KEY}" ${UNINSTALL_KEY} "HelpLink" ${WEB_PAGE}

    ; only allow "Remove" of program
    WriteRegDWORD "${ROOT_KEY}" ${UNINSTALL_KEY} "NoModify" 1
    WriteRegDWORD "${ROOT_KEY}" ${UNINSTALL_KEY} "NoRepair" 1

    ; Estimate size for uninstall entry
    ${GetSize} "$INSTDIR" "/S=0K" $0 $1 $2
    IntFmt $0 "0x%08X" $0
    WriteRegDWORD "${ROOT_KEY}" ${UNINSTALL_KEY} "EstimatedSize" "$0"

    ; Create uninstaller
    WriteUninstaller "$INSTDIR\Uninstall.exe"

SectionEnd

Section -Shortcuts

    ; Create shortcuts
    !insertmacro MUI_STARTMENU_WRITE_BEGIN Application
    CreateDirectory "$SMPROGRAMS\$StartMenuFolder"
    CreateShortCut "$SMPROGRAMS\$StartMenuFolder\${NAME}.lnk" \
        "$INSTDIR\${PROG}" "" "$INSTDIR\${NAME}.ico"
    CreateShortCut "$SMPROGRAMS\$StartMenuFolder\Uninstall.lnk" \
        "$INSTDIR\Uninstall.exe"
    CreateShortCut "$SMPROGRAMS\$StartMenuFolder\ncid.conf.lnk" \
        "$INSTDIR\ncid.conf"
    CreateShortCut "$SMPROGRAMS\$StartMenuFolder\logs.lnk" \
        "${LOGS}"
    CreateShortCut "$SMPROGRAMS\$StartMenuFolder\GPL.txt.lnk" \
        "$INSTDIR\GPL.txt"
    CreateShortCut "$DESKTOP\${NAME}.lnk" "$INSTDIR\${PROG}" "" \
        "$INSTDIR\${NAME}.ico"
    WriteINIStr  "$INSTDIR\${WEB_TITLE}.url" \
                 "InternetShortcut" "URL" "${WEB_PAGE}"
    WriteINIStr  "$SMPROGRAMS\$StartMenuFolder\${WEB_TITLE}.url" \
                 "InternetShortcut" "URL" "${WEB_PAGE}"
    !insertmacro MUI_STARTMENU_WRITE_END

SectionEnd

; selected
Section "Desktop shortcut" Section2
CreateShortCut "$DESKTOP\${NAME}.lnk" "$INSTDIR\${PROG}" "" "$INSTDIR\${NAME}.ico"
SectionEnd

; unselected
Section /o "Startup" Section3
CreateShortCut "$SMPROGRAMS\Startup\${NAME}.lnk" "$INSTDIR\${PROG}" "" "$INSTDIR\${NAME}.ico"
SectionEnd

;--------------------------------
;Descriptions

    ; Language strings
    LangString DESC_Section1 ${LANG_ENGLISH} "NCID (Network Caller ID) Universal Client"
    LangString DESC_Section2 ${LANG_ENGLISH} "Create desktop shortcut to the NCID Universal Client."
    LangString DESC_Section3 ${LANG_ENGLISH} "Launch NCID Universal Client when Windows starts."
    LangString CHANGE_TITLE ${LANG_ENGLISH} "Choose Server Address (Host:Port)"
    LangString CHANGE_SUBTITLE ${LANG_ENGLISH} "Host can be an IP address or a host name."
    LangString VERIFY_TITLE ${LANG_ENGLISH} "Verify Server Address"
    LangString VERIFY_SUBTITLE ${LANG_ENGLISH} "Make sure it is correct."

    !insertmacro MUI_FUNCTION_DESCRIPTION_BEGIN
        !insertmacro MUI_DESCRIPTION_TEXT ${Section1} $(DESC_Section1)
        !insertmacro MUI_DESCRIPTION_TEXT ${Section2} $(DESC_Section2)
        !insertmacro MUI_DESCRIPTION_TEXT ${Section3} $(DESC_Section3)
    !insertmacro MUI_FUNCTION_DESCRIPTION_END

;--------------------------------
;Uninstaller Section

Section "Uninstall"

  ; Terminate the TK App to terminate ncid.tcl
  ; https://lifehacker.com/351056/quick-kill-multiple-programs-with-batch-files
  Exec "taskkill /f /t /im ${TKAPP}"
  Exec "taskkill /f /t /im ${OTKAPP}"

  SetShellVarContext all

  !insertmacro MUI_STARTMENU_GETFOLDER Application $StartMenuFolder

  ReadRegStr $INSTDIR "${ROOT_KEY}" ${INSTALLER_KEY} ""

  ; Remove shortcuts
  Delete   "$DESKTOP\${NAME}.lnk"
  Delete   "$SMPROGRAMS\Startup\${NAME}.lnk"
  RMDir /r "$SMPROGRAMS\$StartMenuFolder"

  ; Remove directories and files
  ;RMDir /r "$INSTDIR"
  RMDir /r "${IMAGES}"
  RMDir /r "${ICONS}"
  RMDir /r "${LIB}"
  RMDir /r "${MSGS}"
  Delete   "$INSTDIR\${PROG}"
  Delete   "${PLUGINS}\${MESSAGE_DIALOG}"
  Delete   "${PLUGINS}\${US_NUMBER_INFO}"
  Delete   "${PLUGINS}\${DISPLAY_NCID_VARIABLES}"
  RMDir    "${PLUGINS}"
  StrCmp $Skipit ${BST_CHECKED} 0 +6
  RMDir /r "${LOGS}"
  RMDir /r "${PLUGINS}"
  RMDir /r "${DOTNCID}"
  Delete   "$INSTDIR\ncid.conf"
  Delete   "$INSTDIR\${NEWCONF}"
  Delete   "$INSTDIR\${LOGO}"
  Delete   "$INSTDIR\${NAME}.ico"
  Delete   "$INSTDIR\${LICENSE_FILE}"
  Delete   "$INSTDIR\${WEB_TITLE}.url"
  Delete   "$INSTDIR\${TZSCRIPT}"
  Delete   "$INSTDIR\Uninstall.exe"
  RMDir    "$INSTDIR"

  ; file not used anymore
  Delete "$APPDATA\ncid.dat"

  ; Remove registry keys
  ;DeleteRegValue "${ROOT_KEY}" ${AUTORUN_KEY} "${NAME}"
  DeleteRegKey "${ROOT_KEY}" ${UNINSTALL_KEY}
  DeleteRegKey "${ROOT_KEY}" ${INSTALLER_KEY}

SectionEnd
