########################################################################
#                       PROCEDURE DEFINITIONS                          #
########################################################################

proc scanFonts {} {
    global fontList currentFont

    # find a fixed-width font and use it
    foreach family [font families] {
        if {[font metrics \"$family\" -fixed]} {
            lappend fontList $family
        }
    }
    # sort and remove duplicates
    set fontList [lsort -dictionary -unique $fontList]

    set currentFont [lindex $fontList 0]
    write_rc_file "fontList " "set fontList \"$fontList\""
}

proc write_rc_file {regexpr command} {
  global rcfile

  if [file exists $rcfile] {
    set id [open $rcfile]
    set data [read $id]
    close $id
    set lines [lrange [split $data "\n"] 0 end-1]
    if {$regexpr == ""} {
      # command from list
      set lines [lsearch -all -inline -not -regexp $lines "$command"]
    } else {
      set index 0
      foreach line $lines {
        if [regexp $regexpr $line] {
          break
        }
        incr index
      }
      if {$index >= [llength $lines]} {
        lappend lines "$command"
      } else {
        lset lines $index "$command"
      }
    }
      set data [join $lines "\n"]
      set id [open $rcfile w]
      puts $id $data
  } else {
    set id [open $rcfile w]
    puts $id $command
  }
  close $id
}

proc doExit {} {
    global force_font_size
    unset force_font_size
    # check if being run from file manager and not launcher-ncid.tcl
    if {[pwd] == "/"} {exit}
}

########################################################################
#                      MAIN ROUTINE STARTS HERE                        #
########################################################################

global force_font_size
global fontList currentFont

set answer [tk_messageBox -message "About to force font size to be $force_font_size.\nMake sure NCID is not running.\nProceed?" -icon warning -type okcancel]
if { $answer != "ok" } {doExit}


if {![info exists force_font_size]} {
   tk_messageBox -message "You cannot run this helper script directly." -icon error -type ok
   doExit
}

set ScriptDir    [file normalize [file dirname [info script]]]
set ToolsDir     $ScriptDir
set MainDir      [file dirname $ToolsDir]

set rcfile       [file join $MainDir .ncid]

set fontList     ""
set currentFont  ""

if [file exists $rcfile] {
   if [file isdirectory $rcfile] {
      tk_messageBox -message "Unable to save data to $rcfile because it is a directory" -icon error -type ok
      return
   }
   set id [open $rcfile]
   set data [read $id]
   close $id

   set lines [split $data "\n"]

    foreach line $lines {
        if [regexp {font\s+create} $line] {
            eval $line
        } elseif [regexp {(:?fontList\
                             )\s+} $line] {
            eval $line
        }
    }
}

if {$fontList == ""} {scanFonts}

if {$currentFont ==""} {
  if {[catch {font configure FixedFontH}]} {
     set currentFont [lindex $fontList 0]
  } else {
     set currentFont [font configure FixedFontH -family]
  }
  tk_messageBox -message "Current font set to $currentFont" -type ok
}


if {[catch {font configure FixedFontH}]} {
    font create FixedFontH -family "$currentFont" -size $force_font_size
    write_rc_file "FixedFontH" \
            "font create FixedFontH [font configure FixedFontH]"
}
if {[catch {font configure FixedFontM}]} {
    font create FixedFontM -family "$currentFont" -size $force_font_size
    write_rc_file "FixedFontM" \
            "font create FixedFontM [font configure FixedFontM]"
}
if {[catch {font configure FixedFontP}]} {
    font create FixedFontP -family "$currentFont" -size $force_font_size
    write_rc_file "FixedFontP" \
            "font create FixedFontP [font configure FixedFontP]"
}

set spinvalH [font configure FixedFontH -size $force_font_size]
set spinvalM [font configure FixedFontM -size $force_font_size]
set spinvalP [font configure FixedFontP -size $force_font_size]

set fonth "[font configure FixedFontH]"
set fontm "[font configure FixedFontM]"
set fontp "[font configure FixedFontP]"

write_rc_file "FixedFontH" "font create FixedFontH $fonth"
write_rc_file "FixedFontM" "font create FixedFontM $fontm"
write_rc_file "FixedFontP" "font create FixedFontP $fontp"

font delete FixedFontH
font delete FixedFontM
font delete FixedFontP

tk_messageBox -message "Preference file updated." -type ok
doExit