# https://wiki.tcl-lang.org/page/AndroWish%3A+Tips+and+Tricks
# original script by Eric Taylor
# customized for NCID by 
#    Todd Andrews <tandrews@users.sourceforge.net>

    if { $tcl_platform(os) != "Windows NT" } { # to test on winxp before going live on android
        cd [file dirname [info script]]
        #move empty "wish" window to the right of console
        wm geometry . +1053+0
    } else {
        console show
    }

    package require Tk

    set _MoveMsgBox +300+150
    set _SaveMsgBox ""

    #set common directories -- there are two places in this script that have the same code
        set _ScriptDir [file normalize [file dirname [info script]]]
        set _ToolsDir  [file join $_ScriptDir android-tools]
        set _MainDir   [file dirname $_ToolsDir]

    set _Version "[file tail [file rootname [info script]]] Version XxXxX\n\n($_ScriptDir)"

    #use fixed font for tk_messageBox
    option add *Dialog.msg.font TKFixedFont

    #plist is like parray but for lists
    source [file join $_ToolsDir plist.tcl]

    proc moveMsgBox {} {
        global _MoveMsgBox _SaveMsgBox
        set _SaveMsgBox [wm geometry .]
        wm geometry . $_MoveMsgBox
    }

    proc restoreMsgBox {} {
        global _SaveMsgBox
        wm geometry . $_SaveMsgBox
    }

    # menu items run the following, they are defined here, not inside the console eval

    proc AboutScript {} {
        global _Version
        moveMsgBox
        tk_messageBox -message $_Version -type ok
        restoreMsgBox
    }

    proc AboutAndroid {} {
        global _ToolsDir
        moveMsgBox
        source [file join $_ToolsDir getAndroidVer.tcl]
        restoreMsgBox
    }

    proc AboutTclTk {} {
        moveMsgBox
        tk_messageBox -message "TCL/TK version [info patchlevel]" -type ok
        restoreMsgBox
    }

    proc AboutDisplayMetrics {} {
        moveMsgBox
        tk_messageBox -message [plist [borg displaymetrics] 2] -type ok
        restoreMsgBox
    }

    proc AboutDetailOS {} {
        moveMsgBox
        tk_messageBox -message [plist [borg osbuildinfo] 2] -type ok
        restoreMsgBox
    }

    if [catch {
        console eval {
            if { ![info exist ::tk::do_scroll] } {
                set menuincr 0 ;# some consoles have an extra item in the edit menu (a font...) AW doesn't, so leave at 0 for now
                set fontsz {times 9}
                pack forget .console .sb .consoleframe

                pack [frame  .frame -bg black] -side left -fill y -ipady 2 -pady 2
                image create photo ncidLogo -format PNG -file images/ncid-192x192.png
                pack [button .frame.ncid -image ncidLogo -command {.menubar.ncid.launch invoke 0}] -side top -fill x
                pack [button .frame.clear -bg blue -fg white -text Clear -command {.menubar.file invoke 2} -font $fontsz] -side top -fill x
                pack [button .frame.smaller   -bg white -text {font -} -font $fontsz -command {.menubar.edit invoke [expr ( $menuincr+6 )];after 100 {.frame.repos invoke}}] -side top -fill x
                pack [button .frame.bigger    -bg white -text {font +}  -font $fontsz -command {.menubar.edit invoke [expr ( $menuincr+5 )];after 100 {.frame.repos invoke}}] -side top -fill x
                pack [button .frame.exit     -bg red -fg white -text Exit  -font $fontsz -command {exiter}] -side top -fill x
                pack [button .frame.repos -bg white -text Bottom -font $fontsz -command {.console see end; .console mark set insert end}] -side top -fill x
                pack [frame  .frame.frame -bg black] -side top  -fill x
                pack [button .frame.frame.m10 -bg gray  -fg white  -font $fontsz   -text {width -} -command {
                    if { [.console cget -width] > 30 } {
                        .console config -width [expr ( [.console cget -width]-10 )]
                    }
                }] -side top -fill x -expand 1
                pack [button .frame.frame.p10 -bg gray  -fg white  -font $fontsz   -text {width +} -command {
                    if { [.console cget -width] < 120 } {
                        .console config -width [expr ([.console cget -width]+10)]
                    }
                }] -side top -fill x -expand 1
                pack [checkbutton .frame.scroll -bg white -fg black  -text scroll  -font $fontsz -variable ::tk::do_scroll] -side top -fill x
                pack .consoleframe -in . -anchor center -expand 1 -fill both -ipadx 0 -ipady 0 -padx 0 -pady 0 -side left
                pack .console -in .consoleframe -anchor center -expand 1 -fill both -ipadx 0 -ipady 0 -padx 1 -pady 1 -side left
                pack .sb -in .consoleframe -anchor center -expand 1 -fill both -ipadx 0 -ipady 0 -padx 1 -pady 1 -side right

                #puts "console width now [.console config -width]   [wm SaveMsgBox .]" ;# hmmm, seems wm SaveMsgBox different across platforms here
                proc exiter {} {
                    set answer [tk_messageBox -message "Are you sure you want to exit?" -icon question -type yesno]
                    if { $answer == "yes" } {
                        exit
                    }
                }
                set ::tk::console::maxLines 1000
                #.menubar add casc -label Extra -menu [menu .menubar.extra -tearoff 0]
                .menubar add casc -label NCID  -menu [menu .menubar.ncid  -tearoff 0]
                .menubar add casc -label About -menu [menu .menubar.about -tearoff 0]

                proc menu+ {head label cmd} {
                    set cmd2 [list consoleinterp eval $cmd]
                    .menubar.$head add command -label $label -command $cmd2 ;# note this is defined outside the console eval... it's in the normal namespace
                }

                #menu+ extra menu menudo

                #set common directories -- there are two places in this script that have the same code
                    set _ScriptDir [file normalize [file dirname [info script]]]
                    set _ToolsDir  [file join $_ScriptDir android-tools]
                    set _MainDir   [file dirname $_ToolsDir]

                .menubar.ncid add casc -label "Launch..."  -menu [menu .menubar.ncid.launch  -tearoff 0]
                menu+ ncid.launch "Debug console OFF"                "wm geometry . +0+0;console hide;source [file join $_MainDir ncid.tcl]"
                menu+ ncid.launch "Debug console ON"                 "wm geometry . +0+0;source [file join $_MainDir ncid.tcl]"

                .menubar.ncid add casc -label "Install..."  -menu [menu .menubar.ncid.install  -tearoff 0]
                menu+ ncid.install "Re-install (keeps all settings)" "moveMsgBox;source [file join $_ToolsDir install-ncid.tcl];restoreMsgBox"
                menu+ ncid.install "Delete homescreen shortcut"      "moveMsgBox;source [file join $_ToolsDir delete-homescreen-shortcut.tcl];restoreMsgBox"

                .menubar.ncid add casc -label "Preference file..."  -menu [menu .menubar.ncid.pref  -tearoff 0]
                menu+ ncid.pref "Delete"                             "moveMsgBox;source [file join $_ToolsDir delete-pref-file.tcl];restoreMsgBox"
                menu+ ncid.pref "Force font size 8"                  "moveMsgBox;source [file join $_ToolsDir force-font-size-8.tcl];restoreMsgBox"

                menu+ ncid "Show IP address"                         "moveMsgBox;source [file join $_ToolsDir getIp.tcl];restoreMsgBox"

                .menubar.ncid add casc -label "Protected .wishrc..."  -menu [menu .menubar.ncid.protected  -tearoff 0]
                menu+ ncid.protected "Backup"                        "moveMsgBox;source [file join $_ToolsDir backup-protected-.wishrc.tcl];restoreMsgBox"
                menu+ ncid.protected "Restore"                       "moveMsgBox;source [file join $_ToolsDir restore-protected-.wishrc.tcl];restoreMsgBox"
                menu+ ncid.protected "Delete"                        "moveMsgBox;source [file join $_ToolsDir delete-protected-.wishrc.tcl];restoreMsgBox"
                menu+ ncid.protected "Create from template"          "moveMsgBox;source [file join $_ToolsDir create-protected-.wishrc.tcl];restoreMsgBox"

                menu+ about "Script version"                         "AboutScript"
                menu+ about "Android version"                        "AboutAndroid"
                menu+ about "TCL/TK version"                         "AboutTclTk"
                menu+ about "Display metrics"                        "AboutDisplayMetrics"
                menu+ about "Detail OS build info"                   "AboutDetailOS"

                proc ::tk::ConsoleOutput {dest string} {
                    set w .console
                    $w insert output $string $dest
                    ::tk::console::ConstrainBuffer $w $::tk::console::maxLines
                    if {$::tk::do_scroll} {$w see insert}
                }
                set ::tk::do_scroll 1
            }
        }

    } err_code] {
        puts $err_code
    }
