<!-- INSTALL-Mac.md - Removable HEADER Start -->

Last edited: Apr 04, 2023

<!-- Removable HEADER End -->

## <a name="instl_mac_top"></a>Macintosh Install

> If NCID does not work, see [INSTALL](#instl_generic_top)
  for some simple tests.

> If you're using a gateway, review the appropriate section in
  [Gateways](#gateways_top).

> [Table of Contents](#doc_top)

### Sections:

> [SYSTEM REQUIREMENTS:](#instl_mac_sr)  
  [COMPILE/INSTALL/UPGRADE from Source:](#instl_mac_comp)  
  [CONFIGURE:](#instl_mac_conf)  
  [FIRST STARTUP:](#instl_mac_fs)  
  [(AUTO)START/STOP:](#instl_mac_ss)  
  [CHECKING DAEMON STATUS:](#instl_mac_check)  
  [TRIMMING LOG FILES:](#instl_mac_trim)  

### <a name="instl_mac_sr"></a>SYSTEM REQUIREMENTS:

> #### macOS:

>>- NCID should work on macOS 11.7 (Big Sur) and later versions.
  It requires an Intel 64bit processor.

> #### Graphical User Interface (GUI):

>>- The native macOS GUI called **Aqua** is not compatible with the NCID GUI client. Instead, it needs **XQuartz**, a macOS specific version of **X Windows** (a.k.a. **X11**). **XQuartz** is installed as a separate application and runs alongside **Aqua**.

>>- In addition to **XQuartz**, a special version of TCL/TK is required that works
with **XQuartz**. The steps to install these depend on which package manager is used and are incorporated below.

> #### Package Manager:

>> Pick *one* of the following package managers (do not install both).

>>##### <u>*Homebrew*</u>:
>>>- This is the preferred package manager because it appears to be more actively maintained, especially for the latest versions of macOS. Check [here](https://docs.brew.sh/Installation#macos-requirements) to see if you meet the minimum system requirements.
>>>
>>>- Click [here](https://brew.sh/) for instructions to download and install Homebrew.
>>>
>>>- As of this writing for NCID 1.14, there is a bug in the Homebrew formula to install the special version of TCL/TK that is required to work with **XQuartz**: You cannot successfully install TCL/TK if the path to the Xcode Command Line Tools (CLT) contains an embedded space. This should only be an issue if you install the full Xcode application to a location other than the default, which is /Applications/Xcode. To determine if this issue applies to your macOS environment, examine the path emitted by the following command typed at a shell prompt:
>>>
>>>             xcode-select -p
>>>

>>##### <u>*MacPorts*</u>:
>>>- You will likely use MacPorts for older versions of macOS that Homebrew no longer supports.  
>>>
>>>- File **/etc/paths** needs to be edited to put in directories for finding MacPorts.
 Use **sudo** to make a backup first, then edit the file with:
>>>
>>>
>>>             sudo vi /etc/paths
>>>
>>>
>>>>> so that it looks like this:
>>>>
>>>
>>>>
        /opt/local/libexec/gnubin  
        /opt/local/bin  
        /opt/local/sbin  
        /usr/bin  
        /bin  
        /usr/sbin  
        /sbin  
        /usr/local/bin  
        /usr/local/sbin  
>>>
>>>- Click [here](https://www.macports.org/install.php) for instructions to download and install MacPorts.

>>>
>>>- A minimum of MacPorts version 2.8.0 is required. However, as of this writing for NCID 1.14, there is a bug in version 2.8.0 and later where the hidapi package will not install header file  hidapi_darwin.h; this causes the compile to fail. This has been reported to the MacPorts maintainers. 

### <a name="instl_mac_comp"></a>COMPILE/INSTALL/UPGRADE from Source:

> #### Prerequisite:

>>- When building NCID from source, you *must* compile on a
  case-sensitive macOS filesystem.
  This requirement is ONLY for compiling NCID. Do not
  attempt to put other macOS applications on a
  case-sensitive filesystem because they will probably
  not work as expected.

>>- Use the **diskutil** command to determine the filesystem type.
Assuming you will be installing to the Mac's startup
volume, i.e., the root filesystem, type the following:  
>
>>          diskutil info / | fgrep -i "file system"

>>- Depending on the version of macOS, the default filesystem type is 'APFS' or 'Journaled HFS+'
 which are not case-sensitive.

>>- Look for 'Case-sensitive' in the output of the **diskutil**
command above. If you don't see it, you can create
a small, case-sensitive disk image just so you can
compile NCID. Approximately 100 megabytes are required
for each version of NCID you intend to compile. A 
minimum 200 megabyte disk image size is recommended.
>
>>          hdiutil create -size 200m \  
>                         -fs "Case-sensitive HFS+" \  
>                         -volname NCID ~/NCID.dmg  

>>- Next, mount the disk image:
>
>>          hdiutil attach ~/NCID.dmg

>>- and change to its directory where you will place the source:
>
>>          cd /Volumes/NCID

>>- Rebooting the Mac computer will normally unmount the disk image
requiring the **hdiutil attach** command to be manually run each time.
You can set it up to automount by doing the following:

>>>1. Login as the user where you want to autmount NCID.dmg.

>>>2. From a Terminal prompt, mount the disk image:<p> **hdiutil attach ~/NCID.dmg**

>>>3. There should now be an icon on the Desktop called NCID showing
that it has been mounted.

>>>4. Go to System Preferences->Users & Groups->Login Items.
Drag the NCID icon from the right edge of the Desktop and drop it onto 
the Login Items (top/bottom/doesn't matter where in the list). 

>>>5. The Login Items window may not refresh automatically so click on
the Password tab and then go back to the Login Items tab. You should 
then see NCID in the list of kind "Volume".

> #### <u>*Homebrew*</u>:

>>- Do not use **sudo** when executing **brew install**.

>>- The following packages are required:
>
>>          brew tap sethrfore/homebrew-r-srf
>>          brew install icu4c libpcap libphonenumber pcre2 hidapi
>>          brew install make wget sethrfore/r-srf/tcl-tk-x11 bwidget
>>          brew install xquartz
>>

>>- Download the source from SourceForge:  
>
>>>
>>>
>>>          wget https://sourceforge.net/projects/ncid\
>>>                  /files/ncid/<version>/ncid-<version>-src.tar.gz


>>> Example:  

>>>          wget https://sourceforge.net/projects/ncid\
>>                  /files/ncid/1.14/ncid-1.14-src.tar.gz

>>- Copy `ncid-<version>-src.tar.gz` to the macOS case-sensitive filesystem and 
  then type:
>
>>          tar -xzvf ncid-<version>-src.tar.gz  
>>  
>>          cd ncid  
>>  
>>          make mac  
>>  
>>          sudo make mac-install  
                        

>>-  Change the default symbolic links for the TCL/TK interpreter to point to the **XQuartz** version by typing the following (this normally only needs to be done once or if Homebrew updates TCL/TK):
>
>>          ncid-setup mac-homebrew-tcl

>>-  Go to Applications->Utilities and launch **XQuartz**. This will create
its default Preference file. It can take a minute for **XQuartz** to finish
running the first time. This only needs to be done once.

>>-  Configure **XQuartz** to run the NCID GUI client when **XQuartz** starts
     by typing the following (only needs to be done once):
>
>>          ncid-setup mac-xquartz-app-menu
>>

> #### <u>*MacPorts*</u>:

>>- You must use **sudo** when executing **port install**.

>>- The following packages are required:
>
>>          sudo port install libpcap libphonenumber-cpp pcre2 gmake 
>>          sudo port install dos2unix wget bwidget hidapi abseil
>>          sudo port install tk +quartz 

>>- Download the source from SourceForge:  
>
>>>
>>>
>>>          wget https://sourceforge.net/projects/ncid\
>>>                  /files/ncid/<version>/ncid-<version>-src.tar.gz


>>> Example:  

>>>          wget https://sourceforge.net/projects/ncid\
>>                  /files/ncid/1.14/ncid-1.14-src.tar.gz

>>- Copy `ncid-<version>-src.tar.gz` to the macOS case-sensitive filesystem and 
  then type:
>
>>
>
>>          tar -xzvf ncid-<version>-src.tar.gz  
>>  
>>          cd ncid  
>>  
>>          make mac  
>>  
>>          sudo make mac-install  
                        
>>-  Go to Applications->Utilities and launch **XQuartz**. This will create
its default Preference file. It can take a minute for **XQuartz** to finish
running the first time. This only needs to be done once.

>>-  Configure **XQuartz** to run the NCID GUI client when **XQuartz** starts
     by typing the following (only needs to be done once):
>
>>          ncid-setup mac-xquartz-app-menu
>>

> #### Other Notes:
> 
>>- For both an install and an upgrade, existing configurations are automatically 
  preserved, but new ones installed will have .new as the extension. You will need to
  manually compare your current configuration files with the ".new" ones and manually
  edit any differences.

>>- Optional gateways will require additional Perl packages before they can be run.
  Installing these with the native Perl package manager called **cpan**.

>>>- This package is required to run email2ncid, obi2ncid, rn2ncid, wc2ncid, wct, and xdmf2ncid:

>>>          cpan install Config::Simple

>>>- This additional package is required to run wc2ncid, wct and xdmf2ncid:

>>>          cpan install Data::HexDump

                           
### <a name="instl_mac_conf"></a>CONFIGURE:

> The Makefile preconfigures ncidd.conf for macOS, but you may
  want to change some of the defaults.

> - If you are using a gateway instead of a local modem,
    you need to set cidinput:  
>  
>>
>
>          set cidinput = 1 # Caller ID from a serial or USB device and optional gateways  
>          set cidinput = 2 # Caller ID from gateways with modem support  
>          set cidinput = 3 # Caller ID from gateways without modem support  

> - If you are using a local modem with or without a gateway:  
>  
>>
>
>          set cidinput = 0  (this is the default)  

### <a name="instl_mac_fs"></a>FIRST STARTUP:

> NCID requires the server and at least one client to function. The
  server is required on one computer or device, but the client can be
  installed on as many computers as needed.

> - If you are running the server and client on the same computer
    and using a modem:  
>  
>>
>
>          sudo /usr/local/sbin/ncidd  
>  
>> In Finder, navigate to Applications->Utilities and double-click on **XQuartz** (or **XQuartz.app**). 

>
> - If you are running the server and using a gateway:  
>  
>>
>
>          sudo /usr/local/sbin/ncidd  
>  
>          sudo /usr/local/sbin/<name of gateway>  
>  
>> In Finder, navigate to Applications->Utilities and double-click on **XQuartz** (or **XQuartz.app**). 

> - Call yourself and see if it works. If not, stop the gateway first (if used) and then 
> stop the server, using **sudo kill** and the appropriate process ID. Continue by
> reading the test sections.

> - If everything is OK, enable the NCID server, gateways and client modules you are
>  using to autostart at boot.

### <a name="instl_mac_ss"></a>(AUTO)START/STOP:
              
> #### SERVER:
    
>> Under macOS the mechanism used to start the NCID
   server processes is **launchd** and requires **.plist** files in
   /Library/LaunchDaemons. The naming convention used is as follows:

>>
>
>          /Library/LaunchDaemons/net.sourceforge.ncid-{name}.plist  
   
>> Appropriate **.plist** files for the NCID server processes are created 
   automatically when NCID is installed, however, they must be manually
   activated.
    
>> Once activated, no action is typically required as the **.plist** files are 
   configured to automatically start each time the system boots.

>> You do not interact with **launchd** directly, instead you use the **launchctl**
   command line utility.

>> You should only activate the NCID servers, gateways and client modules you need. 
   Activating will also start the process immediately; there is no need
   to reboot.

>> The syntax for stopping the daemons is the same as starting them, except
   you use the *unload* subcommand instead of the *load* subcommand. Doing an *unload*
   stops the daemon immediately and prevents it from starting automatically
   the next time the system is booted.
    
>> Here are some examples:

>> - Start the NCID server:
>  
>          sudo launchctl load -w \
>               /Library/LaunchDaemons/net.sourceforge.ncidd.plist  

>> - Stop the sip2ncid server:
>  
>          sudo launchctl unload -w \
>               /Library/LaunchDaemons/net.sourceforge.sip2ncid.plist  

>> - Start ncid with ncid-page:
>  
>          sudo launchctl load -w \
>               /Library/LaunchDaemons/net.sourceforge.ncid-page.plist  

>> Review the man page: **man launchctl**

> #### CLIENT:
    
>> For the NCID GUI client, no **.plist** is currently provided because of the 
   requirement that NCID must be installed as root and the GUI preference file
   is specific to each user.

>> However, if you follow the steps described in this document, **XQuartz** will 
   be configured to run the NCID GUI client when it launches. To have XQuartz 
   launch when you automatically log in, drag **XQuartz** 
   (or **XQuartz.app**) from Applications->Utilities to your account's Login 
   Items: System Preferences->Users & Groups->Login Items tab.

>>  You may optionally want to drag the **XQuartz** icon from Applications->Utilities
and put it in the Dock.

        
### <a name="instl_mac_check"></a>CHECKING DAEMON STATUS:

> Use the **launchctl** *list* subcommand to show the daemons currently loaded, 
  optionally using **fgrep** to filter out only NCID related processes.
    
> Daemons currently running will have a process id.
    
> Daemons which were stopped without an error will not be listed at all.
    
> If a daemon has stopped due to an error, it will have no process id but 
  will have a numeric exit status. Examine the contents of the
  /var/log/system.log file to determine the problem. Once you fix the 
  problem, use the **launchctl** *unload* subcommand followed by the *load* subcommand.

> Example:  
>  
>          sudo launchctl list|fgrep net.sourceforge.ncid  
>  
>          PID     Status  Label  
>          422     -       net.sourceforge.ncid-notify  
>          419     -       net.sourceforge.ncidd  

### <a name="instl_mac_trim"></a>TRIMMING LOG FILES:

> macOS uses **newsyslog** to trim files. To trim the cidcall.log and the
  ciddata.log files, add this entry to /etc/newsyslog.conf
>  
>          /var/log/cid*.log   root:wheel 644 5 * $M1D0 GN
