/*
 * ncid2ncid.h - This file is part of ncid2ncid.
 *
 * Copyright (c) 2005-2022
 * by John L. Chmielewski <jlc@users.sourceforge.net>
 *
 * ncid2ncid is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * ncid2ncid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ncid2ncid.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <unistd.h>
#include <getopt.h>
#include <string.h>
#include <signal.h>
#include <limits.h>
#include <netdb.h>
#include <sys/fcntl.h>
#include <sys/socket.h>
#include <netinet/in_systm.h> /* needed for FreeBSD */
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/udp.h>
#include "version.h"
#include "libncid.h"

#define IDENT       "HELLO: IDENT: "
#define COMMAND     "HELLO: CMD: "

#ifndef PIDFILE
#define PIDFILE     "/var/run/ncid2ncid.pid"
#endif

#ifndef LOGFILE
#define LOGFILE     "/var/log/ncid2ncid.log"
#endif

#ifndef CONFIG
#define CONFIG      "/etc/ncid/ncid2ncid.conf"
#endif

#ifndef HOST_NAME_MAX
#define HOST_NAME_MAX 64 /* HOST_NAME_MAX is POSIX.1, Linux defines it as 64 */
#endif

#define TOHOST      "127.0.0.1"
#define PORT        3333        /* default port for all servers */

#define FATAL       1
#define NOTFATAL    0

#define POLL_TIMEOUT     30 * 1000   /* poll() timeout in milliseconds */

#define NOFROM      "Missing 'FROM' host 1, use"
#define REQOPT      " -f host[:port] || --fromhost host[:port]"

#define NL          '\n'
#define PREFIX      "+"
#define PRETTYINDENT "       "

#define SHOWVER     "%s %s\n%s\n"
#define DESC        "%s - NCID server to NCID server gateway\n"
#define NOOPT       "%s: not an option: %s\n"
#define USAGE       "\
Usage:   %s [options]\n\
Options: [-C <filename>      | --config <filename>]\n\
         [-D                 | --debug]\n\
         [-f <[host][:port]> | --fromhost <[host][:port]>]\n\
         [-H <0|1>           | --hostname-flag <0|1>]\n\
         [-h                 | --help]\n\
         [-L <filename>      | --logfile <filename>]\n\
         [-t <[host][:port]> | --tohost <[host][:port]>]\n\
         [-P <filename>      | --pidfile <filename>]\n\
         [-u                 | --usage]\n\
         [-V                 | --version]\n\
         [-v <1-9>           | --verbose <1-9>]\n\
         [-W 0|1             | --warn <0|1>]\n\
         [--osx-launchd]\n\
"
