/*
 * The following Copyright Notice references the use of the poll
 * stub below. The GPL license is not in effect for the following code snippet
 * braced by the ifdef __MACH__. You are free to embed the following software
 * and copyright notice without the encumberances of the GNU General Public
 * License.
 *
 * Copyright (c) 2002 Mark Salyzyn
 * Copyright (c) 2004, 2024
 * All rights reserved.
 *
 * modified by John L. Chmielewski:
 *  created poll.h and moved lines there, events and revents were reversed,
 *  added switch statement to look at return value of select(), added more
 *  poll() defines, modified logic to find fd that requires service
 *
 * TERMS AND CONDITIONS OF USE
 *
 * Redistribution and use in source form, with or without modification, are
 * permitted provided that redistributions of source code must retain the
 * above copyright notice, this list of conditions and the following
 * disclaimer.
 *
 * This software is provided `as is' by Mark Salyzyn and any express or implied
 * warranties, including, but not limited to, the implied warranties of
 * merchantability and fitness for a particular purpose, are disclaimed. In no
 * event shall Mark Salyzyn be liable for any direct, indirect, incidental,
 * special, exemplary or consequential damages (including, but not limited to,
 * procurement of substitute goods or services; loss of use, data, or profits;
 * or business interruptions) however caused and on any theory of liability,
 * whether in contract, strict liability, or tort (including negligence or
 * otherwise) arising in any way out of the use of this software, even if
 * advised of the possibility of such damage.
 */

#ifdef __MACH__

#include "ncid_poll.h"

int poll(struct pollfd * pfds, unsigned int npfds, int ptimeout)
{
    fd_set          readfds;
    fd_set          writefds;
    fd_set          exceptfds;
    struct timeval  timeout = { ptimeout / 1000, (ptimeout % 1000) * 1000 };
    int             maxfd = 0;
    int             retVal = 0;
    unsigned int    i;

    FD_ZERO(&readfds);
    FD_ZERO(&writefds);
    FD_ZERO(&exceptfds);

    /* Set up fds */
    if (pfds != NULL) {
       for (i = 0; i < npfds; i++) {
          pfds[i].revents = 0;
          if (pfds[i].fd < 0 || pfds[i].fd >= FD_SETSIZE) continue;
          if (pfds[i].fd > maxfd) maxfd = pfds[i].fd;
          if (pfds[i].events & POLLIN)
              FD_SET(pfds[i].fd, &readfds);
          if (pfds[i].events & POLLOUT)
              FD_SET(pfds[i].fd, &writefds);
          if (pfds[i].events & POLLPRI)
              FD_SET(pfds[i].fd, &exceptfds);
       }
    }

    switch (retVal = select(maxfd+1, &readfds, &writefds, &exceptfds,
           (ptimeout > 0) ? &timeout : (struct timeval *)NULL))
    {
        case -1:    /* error */
            /* perror("select()"); */
            break;
        case 0:     /* timeout */
            break;
        default:    /* 1 or more events reported */
            retVal = 0;
            for (i = 0; i < npfds; i++) {
               pfds[i].revents = 0;
               if (pfds[i].fd < 0 || pfds[i].fd >= FD_SETSIZE) continue;
               if (FD_ISSET(pfds[i].fd, &readfds))
                  pfds[i].revents |= POLLIN;
               if (FD_ISSET(pfds[i].fd, &writefds))
                  pfds[i].revents |= POLLOUT;
               if (FD_ISSET(pfds[i].fd, &exceptfds))
                  pfds[i].revents |= POLLPRI;
               if (pfds[i].revents != 0)
                  retVal++;
            }

            break;
    }
    return retVal;
}

#endif /* __MACH__ */
