#!/bin/bash -e

usage="usage: ./create_81-txt.sh [--clobber-everything | --make-everything]"

# --clobber-everything will delete all metadata files that are
# downloaded and/or created, as well as libphonenumber source files.

# --make-everything will do the same as --clobber-everything and then
# force them to be downloaded and/or created.

# --make-everything and --clobber-everything should be passed to
# subsequent scripts that create files.

#########################
# Function declarations #
#########################

check_error() {

   rc=$?
   if [ $rc -ne 0 ] ; then
      echo "Error code $rc"
      echo "Exiting ...."
      exit $rc
   fi

}

abort() {

   # pass optional return code in $1
     rc=$1
     [ -z "$rc" ] && rc=1

   echo "Exiting ...."
   exit $rc

}

download_if_havent_yet() {

   # pass <url> [<optional filename>]
   # as long as the filename is the last part of the url, the
   # <optional filename> does not need to be passed

   if [ $# -ne 1 ] && [ $# -ne 2 ] ; then
      echo "Function '${FUNCNAME[0]}' expects one or two arguments but was passed $#."
      abort
   fi

   url="$1"
   filename_only=`basename "$url"`
   [ -n "$2" ] && filename_only=$2

   if [ ! -f $filename_only ] ; then
      echo "Downloading '$filename_only'..."
      wget2 -N --no-verbose --no-hsts --force-progress "$url"
      rc=$?
      if [ $rc -ne 0 ] || [ ! -f $filename_only ] ; then
         echo "Could not download '$url'"
         echo "Maybe it was moved? See README-`pwd` for more info."
         abort -1
      fi
   fi

   return 0

}

abort_if_file_not_found_or_zero_bytes() {

   # pass as many filenames as desired to $1, $2, $3, etc.

   [ $# -eq 0 ] && echo "Function '${FUNCNAME[0]}' needs one or more filenames but wasn't passed any." && abort

   abort_flag=0

   for ifile in $@
      do
        if [ ! -f "$ifile" ] ; then
           echo "File '$ifile' was not found."
           abort_flag=1
        elif [ ! -s "$ifile" ] ; then
           echo "File '$ifile' was created but has zero bytes."
           abort_flag=1
        fi
   done

   [ $abort_flag -eq 1 ] && abort

   return 0

}

################
# MAIN ROUTINE #
################

this_script=$0
this_script_base=`basename "$this_script"`
this_script_base_no_ext=`basename "$this_script" .sh`
this_script_dir=`pwd` # save full path of the directory where THIS script is running so we'll never lose our way

clobber_or_make_arg="$1"

action=GENERATING
if [ $# -gt 1 ] ; then
   echo "Too many arguments passed to $this_script."
   echo "$usage"
   abort
elif [ $# -eq 1 ] ; then
   clobber_or_make_arg="$1"
   if [ $clobber_or_make_arg = "--clobber-everything" ] ; then
      action=CLOBBERING
   elif [ $clobber_or_make_arg = "--make-everything" ] ; then
      action=$action # do nothing
   else
      echo "Invalid argument passed to $this_script."
      echo "$usage"
      abort
   fi
fi

title="${action} JAPANESE METADATA"
dashes=`echo "$title" | tr '[:print:]' '-'`
echo
echo "$dashes"
echo "$title"
echo "$dashes"
echo

result_file=81.txt
result_file_en=${result_file}_en
result_file_ja=${result_file}_ja

# tmp1=${this_script_base_no_ext}-1.tmp
# rm -f *.tmp 

rm -f ${result_file}* *.csv keitai_phs*

if [ -n "$clobber_or_make_arg" ] ; then

   rm -f *.xls *.xlsx

   [ $clobber_or_make_arg = "--clobber-everything" ] && exit 0

fi

# check for required directories and/or scripts
   abort_flag=0

   cd ..
   metadata_generation_dir=`pwd` # saves full path

   # make sure libphonenumber is installed
   x=${metadata_generation_dir}/resources/carrier
        if [ ! -d $x ] ; then
           echo "Required directory '$x' not found."
           echo "This means 'libphonenumber' does not appear to be installed."
           echo "See ${metadata_generation_dir}/README-`basename ${metadata_generation_dir}` for more info."
           abort_flag=1
        fi

   # make sure we can find xls2csv.sh
   x=${metadata_generation_dir}/xls2csv.sh
        if [ ! -x $x ] ; then
           echo "Required script '$x' not found."
           abort_flag=1
        fi

[ $abort_flag -eq 1 ] && abort

cd $this_script_dir


# in case xls files are not found  , It means they have  been updated and the file names have changed  , have a look at 
# https://www.soumu.go.jp/main_sosiki/joho_tsusin/top/tel_number/number_shitei.html
# modify accordingly 
# xls files can change to xlsx as well 


#name of xls files  as feb 2021 

residential_number_1="000697543"
residential_number_2="000697544"
residential_number_3="000697545"
residential_number_4="000697546"
residential_number_5="000697548"
residential_number_6="000697549"
residential_number_7="000697550"
residential_number_8="000697551"
residential_number_9="000697552"

keitai_and_PHS_070="000697563"
keitai_and_PHS_080="000697565"
keitai_and_PHS_090="000697567"

#Residential Numbers
download_if_havent_yet https://www.soumu.go.jp/main_content/${residential_number_1}.xls
download_if_havent_yet https://www.soumu.go.jp/main_content/${residential_number_2}.xls
download_if_havent_yet https://www.soumu.go.jp/main_content/${residential_number_3}.xls
download_if_havent_yet https://www.soumu.go.jp/main_content/${residential_number_4}.xls
download_if_havent_yet https://www.soumu.go.jp/main_content/${residential_number_5}.xls
download_if_havent_yet https://www.soumu.go.jp/main_content/${residential_number_6}.xls
download_if_havent_yet https://www.soumu.go.jp/main_content/${residential_number_7}.xls
download_if_havent_yet https://www.soumu.go.jp/main_content/${residential_number_8}.xls
download_if_havent_yet https://www.soumu.go.jp/main_content/${residential_number_9}.xls

#Keitai  and PHS
download_if_havent_yet https://www.soumu.go.jp/main_content/${keitai_and_PHS_070}.xlsx
download_if_havent_yet https://www.soumu.go.jp/main_content/${keitai_and_PHS_080}.xlsx
download_if_havent_yet https://www.soumu.go.jp/main_content/${keitai_and_PHS_090}.xlsx

echo "Started conversion of official Japanese residential numbers xls files into csv..."

${metadata_generation_dir}/xls2csv.sh \
              ${residential_number_1}.xls ${residential_number_2}.xls ${residential_number_3}.xls ${residential_number_4}.xls \
              ${residential_number_5}.xls ${residential_number_6}.xls ${residential_number_7}.xls ${residential_number_8}.xls \
              ${residential_number_9}.xls

abort_if_file_not_found_or_zero_bytes ${residential_number_1}.csv ${residential_number_2}.csv ${residential_number_3}.csv ${residential_number_4}.csv ${residential_number_5}.csv ${residential_number_6}.csv ${residential_number_7}.csv ${residential_number_8}.csv ${residential_number_9}.csv

echo "Started conversion of official Japanese Keitai and PHS  numbers xls files into csv..."

${metadata_generation_dir}/xls2csv.sh xlsx \
              ${keitai_and_PHS_070}.xlsx ${keitai_and_PHS_080}.xlsx \
              ${keitai_and_PHS_090}.xlsx


abort_if_file_not_found_or_zero_bytes ${keitai_and_PHS_070}.csv ${keitai_and_PHS_080}.csv ${keitai_and_PHS_090}.csv

echo "Finished conversion of official Japanese carrier xls files into csv."
echo "Converting to Google metadata format..."

echo "Processing Keitai and PHS..."
cat ${keitai_and_PHS_070}.csv | grep "^070" | awk -F "\"*,\"*" '{print $1 "0|" $2 "\n" $1 "1|" $3 "\n" $1 "2|" $4"\n" $1 "3|" $5 "\n" $1 "4|" $6 "\n" $1 "5|" $7 "\n" $1 "6|" $8 "\n" $1 "7|" $9 "\n" $1 "8|" $10 "\n" $1 "9|" $11}' > keitai_phs_070.txt
cat ${keitai_and_PHS_080}.csv | grep "^080" | awk -F "\"*,\"*" '{print $1 "0|" $2 "\n" $1 "1|" $3 "\n" $1 "2|" $4"\n" $1 "3|" $5 "\n" $1 "4|" $6 "\n" $1 "5|" $7 "\n" $1 "6|" $8 "\n" $1 "7|" $9 "\n" $1 "8|" $10 "\n" $1 "9|" $11}' > keitai_phs_080.txt
cat ${keitai_and_PHS_090}.csv | grep "^090" | awk -F "\"*,\"*" '{print $1 "0|" $2 "\n" $1 "1|" $3 "\n" $1 "2|" $4"\n" $1 "3|" $5 "\n" $1 "4|" $6 "\n" $1 "5|" $7 "\n" $1 "6|" $8 "\n" $1 "7|" $9 "\n" $1 "8|" $10 "\n" $1 "9|" $11}' > keitai_phs_090.txt

#remove empty blocks
cat keitai_phs_070.txt keitai_phs_080.txt keitai_phs_090.txt |
    grep -v "|$" |
    sed -E "s/^0/81/" > keitai_phs.txt


echo "Processing residential numbers..."

cat ${residential_number_1}.csv ${residential_number_2}.csv ${residential_number_3}.csv ${residential_number_4}.csv ${residential_number_5}.csv ${residential_number_6}.csv ${residential_number_7}.csv ${residential_number_8}.csv ${residential_number_9}.csv  >  residential.csv

# "使用中" = "using"
grep "使用中" residential.csv |
    sort |
    awk -F "\"*,\"*" '{print $2 "|" $5}' |
    grep "^0" |
    sed -E "s/^0/81/" > 81.txt_ja_data

abort_if_file_not_found_or_zero_bytes 81.txt_ja_data

# add keitai and PHS
cat keitai_phs.txt >> 81.txt_ja_data
#   "使用不可"  = "usage forbiden"
sort 81.txt_ja_data | grep -v  "使用不可" > 81.txt_ja_data_sorted
cat libphonenumber-and-ncid-comments.txt 81.txt_ja_data_sorted   > $result_file_ja

abort_if_file_not_found_or_zero_bytes 81.txt_ja_data_sorted $result_file_ja

echo "Resulting Japanese language version of file '$result_file' has `cat $result_file_ja |wc -l` lines"

echo "Translating Japanese language version to English..."

cat $result_file_ja |
    sed -e "s/Ｃｏｌｔテクノロジーサービス株式会社/Colt Technology Service/" |
    sed -e "s/Ｃｏｌｔテクノロジーサービス/Colt Technology Service/" |

	sed -e "s/エヌ・ティ・ティ・コミュニケーションズ株式会社/NTT Communications/" |
    sed -e "s/ＮＴＴコミュニケーションズ/NTT Communications/" |

	sed -e "s/東日本電信電話株式会社/NTT East Japan/" |
    sed -e "s/ＮＴＴ東日本/NTT East Japan/" |

    sed -e "s/西日本電信電話株式会社/NTT West Japan/" |
    sed -e "s/ＮＴＴ西日本/NTT West Japan/" |
    
	sed -e "s/株式会社アイ・ピー・エス/IPS/" |
	sed -e "s/アイ・ピー・エス/IPS/" |

	sed -e "s/ＫＤＤＩ株式会社/KDDI/" |
    sed -e "s/ＫＤＤＩ/KDDI/" |

	sed -e "s/アルテリア・ネットワークス株式会社/Arteria Networks/" |
    sed -e "s/アルテリア・ネットワークス/Arteria Networks/" |

    sed -e "s/株式会社エネルギア・コミュニケーションズ/Energea Communications/" |
    sed -e "s/エネルギア・コミュニケーションズ/Energea Communications/" |

    sed -e "s/株式会社オプテージ/Optage/" |
    sed -e "s/オプテージ/Optage/" |

    sed -e "s/株式会社ジュピターテレコム/Jupiter Telecom/" |
    sed -e "s/ジュピターテレコム/Jupiter Telecom/" |

	sed -e "s/ソフトバンク株式会社/Softbank/" |
    sed -e "s/ソフトバンク/Softbank/" |

    sed -e "s/株式会社三通/Three/" |
    sed -e "s/三通/Three/" |
    
	sed -e "s/中部テレコミュニケーション株式会社/Chubu Telecommunications/" |
    sed -e "s/中部テレコミュニケーション/Chubu Telecommunications/" |

    sed -e "s/株式会社日本緊急通報サービス/Japan emergency call service/" |
    sed -e "s/株日本緊急通報サービス/Japan emergency call service/" |

    sed -e "s/東京テレメッセージ/Tokyo Telemessage/" |

    sed -e "s/東北インテリジェント通信株式会社/Tohoku Intelligent Communication/" |
    sed -e "s/東北インテリジェント通信/Tohoku Intelligent Communication/" |
    
	sed -e "s/楽天コミュニケーションズ/Rakuten Communications/" |

	sed -e "s/株式会社ＮＴＴドコモ/NTT DOCOMO/" |
	sed -e "s/ＮＴＴドコモ/NTT DOCOMO/" |
    
	sed -e "s/沖縄セルラー電話株式会社/Okinawa Cellular/" |
	sed -e "s/沖縄セルラー電話/Okinawa Cellular/" |

	sed -e "s/大江戸テレコム株式会社/Oedo Telecom/" |
	
	sed -e "s/ＺＩＰＴｅｌｅｃｏｍ株式会社/Zip Telecom/" |

    sed -e "s/株式会社ＳＴＮｅｔ/STNet/" |
    
	sed -e "s/株式会社ＱＴｎｅｔ/QTNet Co./" |
	
	sed -e "s/楽天モバイル株式会社/Rakuten Mobile/" |
    sed -e "s/楽天モバイル/Rakuten Mobile/" > $result_file_en

abort_if_file_not_found_or_zero_bytes $result_file_en

echo "Resulting English language version of file '$result_file' has `cat $result_file_en |wc -l` lines"

mkdir -p ${metadata_generation_dir}/resources/carrier/ja
check_error

cp $result_file_en  ${metadata_generation_dir}/resources/carrier/en/$result_file
check_error
cp $result_file_ja  ${metadata_generation_dir}/resources/carrier/ja/$result_file
check_error

#awk -F "|" '{print $2 }' 81.txt_jp_data |
#sort |
#uniq  >  jp_carrier.txt

echo "Script completed successfully."

# all conversion scripts need to do 'exit 0' if successful so that their calling script (if any) will not abort
exit 0
