#!/bin/bash

# ncid-kpopup
# usage: ncid --no-gui --module ncid-kpopup

# Created by Randy L. Rasmussen on Thu Dec 20, 2007

# Last edited: Dec 8, 2020

# Copyright (c) 2007-2018 by
#   Randy L. Rasmussen
#   John L. Chmielewski <jlc@users.sourceforge.net>

# Display a popup caption and speak the caller id
# Requires kdialog (for popup) and festival (to speak)

ConfigDir=/etc/ncid/conf.d
ConfigFile=$ConfigDir/ncid-kpopup.conf

# WARNING: module ncid-kpopup may call ncid-speak
#          be sure ncid-kpopup passes stdin correctly to ncid-speak

ncid_speak=/usr/share/ncid/ncid-speak

### defaults if not using config file ###
###  see $ConfigFile for description  ###
allowed_types="ALLTYPES"
kdialog=/usr/bin/kdialog
kpopup_geo="0x0+1600+1000" #Display in the bottom right corner of a 22" monitor
kpopup_timeout=10 #Displays popup for X number of seconds used by kdialog
kpopup_speak=""

[ -f $ConfigFile ] && . $ConfigFile

# backward compatibility for pre-NCID 1.8 config files
  [ -n "$kpopup_types" ] && allowed_types="$kpopup_types"

#SEDCOMMONSH# at compile time, file ncid-modules-common.sh will be inserted here

# Exit if $TYPE not found
[ -z "$found" ] && exit 0

# set notification title
  title="${type_desc}:"

if [ "$type_cat" = "MSGTYPE" ]
then
    $kdialog --geometry $kpopup_geo --title "$title" --passivepopup \
         "$MESG" $kpopup_timeout &
else
    $kdialog --geometry $kpopup_geo --title "$title" --passivepopup \
         "$TYPE $NAME $FNMBR" $kpopup_timeout &

    # this speaks if there is a match
    if [ "$kpopup_speak" = "enable" ]
    then
        # Added the following to unmute Line in for kmix if muted
        muted=$(dcop kmix Mixer0 mute 2)
        if [ "$muted" = "true" ] #If volume is muted
        then
        dcop kmix Mixer0 toggleMute 2
        fi

        # call the ncid-speak module
        echo -e "$TYPE\n$DATE\n$TIME\n$LINE\n$FNMBR\n$NAME\n$MTYPE\n$MESG\n" | $ncid_speak
    fi
fi

exit 0
