# === Start of common module script routines - ncid-modules-common.sh

# Last edited: Jun 30, 2022

# Before this code is executed:
#    - set $allowed_types as desired
#
# After this code is executed:
#    - stdin will have been read into $DATE, $TIME, $NMBR, $NAME, etc.
#    - $found=1 if $TYPE is in $allowed_types; otherwise $found will be null
#    - $type_cat(egory) will be "CALLTYPE", "MSGTYPE", or "UNKNOWNTYPE"
#    - $type_smart_phone is "Y" for both smart phone calls and messages,
#      otherwise $type_smart_phone will be null
#    - $type_desc(ription) will be set to human readable verbiage

# $allowed_types can be:
#    - any mix of zero or more individual call/line types separated by spaces
#    - any mix of zero or more type categories separated by spaces
#    - the special category "ALLTYPES"
#    - null, which defaults to "ALLTYPES"
#
# If $allowed_types includes spaces, you must surround the list with quotes.
#
# Note that "ALLTYPES" causes **any** $TYPE to be accepted.
#
# Examples:
#    allowed_types="CALLTYPE"
#    allowed_types="CALLTYPE MSGTYPE"
#    allowed_types="CID MSGTYPE HUP"

# input is always 12 lines
INPUT="DATE TIME NMBR NAME LINE TYPE MESG MTYPE FNMBR CTRY LOCA CARI"
for i in $INPUT; do read $i; done

# if input is from a call:
# input: DATE\nTIME\nNMBR\nNAME\nLINE\nTYPE\n""\n""\nFNMBR\nCTRY\nLOCA\nCARI\n
#
# if input is from a message
# input: DATE\nTIME\nNMBR\nNAME\nLINE\nTYPE\nMESG\nMTYPE\nFNMBR\nCTRY\nLOCA\nCARI\n

# determine $TYPE category and description
  type_cat="UNKNOWNTYPE"
  type_desc="Unknown Call Type ($TYPE)"
  type_smart_phone=

case "$TYPE" in

  BLK) type_cat=CALLTYPE;  type_smart_phone= ; type_desc="Blacklisted Call Blocked";;
  CID) type_cat=CALLTYPE;  type_smart_phone= ; type_desc="Incoming Call";;
  HUP) type_cat=CALLTYPE;  type_smart_phone= ; type_desc="Blacklisted Call Hangup";;
  MWI) type_cat=CALLTYPE;  type_smart_phone= ; type_desc="Voicemail Message Waiting";;
  OUT) type_cat=CALLTYPE;  type_smart_phone= ; type_desc="Outgoing Call";;
  PID) type_cat=CALLTYPE;  type_smart_phone=Y; type_desc="Caller ID from a smart phone";;
  PUT) type_cat=CALLTYPE;  type_smart_phone=Y; type_desc="Outgoing Smart Phone Call";;
  RID) type_cat=CALLTYPE;  type_smart_phone= ; type_desc="Ringback Call";;
  WID) type_cat=CALLTYPE;  type_smart_phone= ; type_desc="Call Waiting Caller ID";;

  MSG)  type_cat=MSGTYPE;  type_smart_phone= ; type_desc="Message";;
  NOT)  type_cat=MSGTYPE;  type_smart_phone=Y; type_desc="Notice of a smart phone message";;

esac

# Is this $TYPE one that can be processed by this module?
found=
if [ -z "$allowed_types" ]
then
    allowed_types="ALLTYPES"
    found=1
else
    for i in $allowed_types
    do
        if   [ $i = "ALLTYPES" ];  then found=1; break;
        elif [ $i = "$type_cat" ]; then found=1; break;
        elif [ $i = "$TYPE" ];     then found=1; break;
        fi
    done
fi

# === End of common module script routines - ncid-modules-common.sh
