#!/bin/bash

# ncid-page
# usage: ncid --no-gui --module ncid-page

# Last edited: Dec 8, 2020

# Copyright (c) 2001-2018 by
#   John L. Chmielewski <jlc@users.sourceforge.net>

# sends Caller ID or message to a cell phone, pager, or any other email address
# Requires a mail program

ConfigDir=/etc/ncid/conf.d
ConfigFile=$ConfigDir/ncid-page.conf

### defaults if not using config file ###
###  see $ConfigFile for description  ###
allowed_types="CID MSG"
PageMail=mail
PageTo=
PageFrom=mail
PageOpt=

[ -f $ConfigFile ] && . $ConfigFile

# backward compatibility for pre-NCID 1.8 config files
  [ -n "$PageTypes" ] && allowed_types=$PageTypes

#SEDCOMMONSH# at compile time, file ncid-modules-common.sh will be inserted here

# Exit if $TYPE not found
[ -z "$found" ] && exit 0

if [ "$type_smart_phone" = "Y" ]
   then 
      exit 0
fi

[ -z "PageTo" ] && \
{
    echo "Must set PageTo to SMS Gateway or other email address"
    exit 1
}

if [ -n "$PageFrom" ]
then
    # if valid user and script ID is root, set rootID
    [ "`id -nu $PageFrom 2> /dev/null`" = "$PageFrom" ] &&
    [ "`id -nu`" = "root" ] && rootID=1
fi

# if line indicator = "" make it "-"
[ -z "$LINE" ] && LINE="-"

if [ "$type_cat" = "MSGTYPE" ]
then
    MailMsg="NCID TYPE: $TYPE\nDATE: $DATE\nTIME: $TIME\nLINE: $LINE\nNMBR: $FNMBR\nNAME: $NAME\n$MTYPE\n$MESG\n"
else
    MailMsg="NCID TYPE: $TYPE\nDATE: $DATE\nTIME: $TIME\nLINE: $LINE\nNMBR: $FNMBR\nNAME: $NAME\n"
fi

if [ -n "$PageOpt" ]
then
    case $PageOpt in
        NAME) PageOpt="-s $NAME";;
        NMBR) PageOpt="-s $FNMBR";;
        NONA) PageOpt="-s $FNMBR $NAME";;
        NANO) PageOpt="-s $NAME $FNMBR";;
        *) PageOpt="";;
    esac
fi

if [ -n "$rootID" ]
then
    # send mail as user $PageFrom
    echo -e $MailMsg |
        su -s /bin/sh -c "$PageMail \"$PageOpt\" $PageTo" $PageFrom
else
    # send mail as user running script
    echo -e $MailMsg | $PageMail "$PageOpt" $PageTo
fi

exit 0
