#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

char * progname;

/*
 * 2's complement 8-bit checksum
 * Modified from: https://stackoverflow.com/questions/3463976/c-file-checksum
 */
 unsigned char checksum (unsigned char *ptr, size_t size) {
 unsigned int sum = 0;

  printf("Checksum data:\n");
  while (size-- != 0)
  {
    printf("%.2x", *ptr);
    sum += *ptr++;
  }
  //jlc: sum = (sum ^ 0xFF) + 1;
  sum = sum % 256;
  return (unsigned char)sum;
}

/*
 * Convert hex string to unsigned characters
 */
void hex2uchar(const char * buf, unsigned char * ucbuf)
{
    int buf_len = strlen(buf)/2, i;
    char tmpchar[5];

    //jlc: bzero(tmpchar, sizeof(tmpchar));
    for (i = 0; i < buf_len; i++)
    {
        strncpy(tmpchar, buf + 2 * i, 2);
        ucbuf[i] = (unsigned char) strtol(tmpchar, NULL, 16);
    }
}

/*
 * hexdump from http://stackoverflow.com/a/29865
 * by user 'epatel' dated Aug 27 2008
 */
void hexdump(const void *ptr, int buflen) {
  const unsigned char *buf = (const unsigned char*)ptr;
  char msgbuf[BUFSIZ];
  char tmpbuf[BUFSIZ];
  int i, j;
  for (i=0; i<buflen; i+=16) {
    sprintf(msgbuf,"%06x: ", i);
    for (j=0; j<16; j++)
      if (i+j < buflen) {
        sprintf(tmpbuf,"%02x ", buf[i+j]);
        strcat(msgbuf,tmpbuf);
      } else strcat(msgbuf,"   ");
    strcat(msgbuf," ");
    for (j=0; j<16; j++)
      if (i+j < buflen) {
        sprintf(tmpbuf,"%c", isprint(buf[i+j]) ? buf[i+j] : '.');
        strcat(msgbuf,tmpbuf);
      }
    strcat(msgbuf,"\n");
    //jlc: logMsg(HEXLEVEL,msgbuf);
    printf("%s", msgbuf);
  }
  printf("\n");
}

int main(int argc, char* argv[])
{
    FILE* fp;
    unsigned char buf[BUFSIZ], ucbuf[BUFSIZ], chksum;
    size_t msglen;

    /* global containing name of program */
    progname = strrchr(argv[0], (int) '/');
    progname = progname ? progname + 1 : argv[0];

    if (argc != 2)
    {
        printf("Usage: %s <file>\n", progname);
        printf("    where <file> = test-checksum-goodmdmf, test-checksum-badexample, ...\n");
        printf("argc = %d, must be 2\n", argc);
        exit(1);
    }
    else
    {
        printf("file: %s\n", argv[argc - 1]);
    }

    fp = fopen(argv[argc - 1], "r");
    if (fp == NULL) {
      perror("Failed: ");
      return 1;
    }

    while (fgets(buf, BUFSIZ, fp))
    {
        // Remove trailing newline
        buf[strcspn(buf, "\n")] = 0;
        if (isdigit(buf[0]))
        {
            printf("XDMF string:\n%s\n\n", buf);
            hex2uchar(buf, ucbuf);
        }
    }

    bzero(ucbuf, BUFSIZ);
    hex2uchar(buf, ucbuf);
    hexdump(ucbuf, strlen(buf) / 2);
    msglen = ucbuf[1];

    /* zero 2 digit checksum value (0x00) = checksum calculated value added to checksum byte */
    chksum = checksum(ucbuf, msglen + 2) + ucbuf[msglen + 2] & 0xff;

    printf("\n\n");
    printf("XDMF string length = %d (0x%.2x)\n", strlen(ucbuf), strlen(ucbuf));
    printf("XDMF data length = %d (0x%.2x)\n", msglen, msglen);
    printf("Checksum data length: %d (0x%.2x)\n", msglen + 2, msglen + 2);
    printf("Checksum byte = %d (0x%.2x)\n", *(ucbuf + msglen + 2), *(ucbuf + msglen + 2));
    printf("checksum = 0x%.2x: %s\n", chksum, chksum ? "bad" : "good", chksum);

    fclose(fp);
    return 0;
}
